/* File: cmd6.c */

/*
 * Copyright (c) 1997 Ben Harrison, James E. Wilson, Robert A. Koeneke
 *
 * This software may be copied and distributed for educational, research,
 * and not for profit purposes provided that this copyright and statement
 * are included in all such copies.  Other copyrights may also apply.
 */

#include "angband.h"

#include "script.h"


/*
 * This file includes code for eating food, drinking potions,
 * reading scrolls, aiming wands, using staffs, zapping rods,
 * and activating artifacts.
 *
 * In all cases, if the player becomes "aware" of the item's use
 * by testing it, mark it as "aware" and reward some experience
 * based on the object's level, always rounding up.  If the player
 * remains "unaware", mark that object "kind" as "tried".
 *
 * This code now correctly handles the unstacking of wands, staffs,
 * and rods.  Note the overly paranoid warning about potential pack
 * overflow, which allows the player to use and drop a stacked item.
 *
 * In all "unstacking" scenarios, the "used" object is "carried" as if
 * the player had just picked it up.  In particular, this means that if
 * the use of an item induces pack overflow, that item will be dropped.
 *
 * For simplicity, these routines induce a full "pack reorganization"
 * which not only combines similar items, but also reorganizes various
 * items to obey the current "sorting" method.  This may require about
 * 400 item comparisons, but only occasionally.
 *
 * There may be a BIG problem with any "effect" that can cause "changes"
 * to the inventory.  For example, a "scroll of recharging" can cause
 * a wand/staff to "disappear", moving the inventory up.  Luckily, the
 * scrolls all appear BEFORE the staffs/wands, so this is not a problem.
 * But, for example, a "staff of recharging" could cause MAJOR problems.
 * In such a case, it will be best to either (1) "postpone" the effect
 * until the end of the function, or (2) "change" the effect, say, into
 * giving a staff "negative" charges, or "turning a staff into a stick".
 * It seems as though a "rod of recharging" might in fact cause problems.
 * The basic problem is that the act of recharging (and destroying) an
 * item causes the inducer of that action to "move", causing "o_ptr" to
 * no longer point at the correct item, with horrifying results.
 *
 * Note that food/potions/scrolls no longer use bit-flags for effects,
 * but instead use the "sval" (which is also used to sort the objects).
 */






/*
 * Eat some food (from the pack or floor)
 */
void do_cmd_eat_food(int h_item)
{
	int item, lev;
	bool ident;

	object_type *o_ptr;

	cptr q, s;


	/* Are we being called from "do_cmd_handle()? */
	if (h_item != INVEN_TOTAL)
	{
		item = h_item;
	}
	else
	{
		/* Restrict choices to food */
		item_tester_tval = TV_FOOD;

		/* Get an item */
		q = "Eat which item? ";
		s = "You have nothing to eat.";
		if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
	}

	/* Get the item (in the pack) */
	if (item >= 0)
	{
		o_ptr = &inventory[item];
	}

	/* Get the item (on the floor) */
	else
	{
		o_ptr = &o_list[0 - item];
	}


	/* Sound */
	sound(MSG_EAT);


	/* Take a turn */
	p_ptr->energy_use = 100;

	/* Identity not known yet */
	ident = FALSE;

	/* Object level */
	lev = k_info[o_ptr->k_idx].level;

	/* Eat the food */
	use_object(o_ptr, &ident);

	/* Combine / Reorder the pack (later) */
	p_ptr->notice |= (PN_COMBINE | PN_REORDER);

	/* We have tried it */
	object_tried(o_ptr);

	/* The player is now aware of the object */
	if (ident && !object_aware_p(o_ptr))
	{
		object_aware(o_ptr);
		gain_exp((lev + (p_ptr->lev / 2)) / p_ptr->lev);
	}

	/* Window stuff */
	p_ptr->window |= (PW_INVEN | PW_EQUIP);


	/* Destroy a food in the pack */
	if (item >= 0)
	{
		inven_item_increase(item, -1);
		inven_item_describe(item);
		inven_item_optimize(item);
	}

	/* Destroy a food on the floor */
	else
	{
		floor_item_increase(0 - item, -1);
		floor_item_describe(0 - item);
		floor_item_optimize(0 - item);
	}
}




/*
 * Quaff a potion (from the pack or the floor)
 */
void do_cmd_quaff_potion(int h_item)
{
	int item, lev;
	bool ident;
	object_type *o_ptr;
	cptr q, s;


	/* Are we being called from "do_cmd_handle()? */
	if (h_item != INVEN_TOTAL)
	{
		item = h_item;
	}
	else
	{
		/* Restrict choices to potions */
		item_tester_tval = TV_POTION;

		/* Get an item */
		q = "Quaff which potion? ";
		s = "You have no potions to quaff.";
		if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
	}

	/* Get the item (in the pack) */
	if (item >= 0)
	{
		o_ptr = &inventory[item];
	}

	/* Get the item (on the floor) */
	else
	{
		o_ptr = &o_list[0 - item];
	}


	/* Sound */
	sound(MSG_QUAFF);


	/* Take a turn */
	p_ptr->energy_use = 100;

	/* Not identified yet */
	ident = FALSE;

	/* Object level */
	lev = k_info[o_ptr->k_idx].level;

	/* Quaff the potion */
	use_object(o_ptr, &ident);

	/* Combine / Reorder the pack (later) */
	p_ptr->notice |= (PN_COMBINE | PN_REORDER);

	/* The item has been tried */
	object_tried(o_ptr);

	/* An identification was made */
	if (ident && !object_aware_p(o_ptr))
	{
		object_aware(o_ptr);
		gain_exp((lev + (p_ptr->lev / 2)) / p_ptr->lev);
	}

	/* Window stuff */
	p_ptr->window |= (PW_INVEN | PW_EQUIP);

	/* Destroy a potion in the pack */
	if (item >= 0)
	{
		inven_item_increase(item, -1);
		inven_item_describe(item);
		inven_item_optimize(item);
	}

	/* Destroy a potion on the floor */
	else
	{
		floor_item_increase(0 - item, -1);
		floor_item_describe(0 - item);
		floor_item_optimize(0 - item);
	}
}


/*
 * Read a scroll (from the pack or floor).
 *
 * Certain scrolls can be "aborted" without losing the scroll.  These
 * include scrolls with no effects but recharge or identify, which are
 * cancelled before use.  XXX Reading them still takes a turn, though.
 */
void do_cmd_read_scroll(int h_item)
{
	int item, used_up, lev;
	bool ident;

	object_type *o_ptr;

	cptr q, s;


	/* Check some conditions */
	if (p_ptr->blind)
	{
		msg_print("You can't see anything.");
		return;
	}
	if (no_lite())
	{
		msg_print("You have no light to read by.");
		return;
	}
	if (p_ptr->confused)
	{
		msg_print("You are too confused!");
		return;
	}


	/* Are we being called from "do_cmd_handle()? */
	if (h_item != INVEN_TOTAL)
	{
		item = h_item;
	}
	else
	{
		/* Restrict choices to scrolls */
		item_tester_tval = TV_SCROLL;

		/* Get an item */
		q = "Read which scroll? ";
		s = "You have no scrolls to read.";
		if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
	}

	/* Get the item (in the pack) */
	if (item >= 0)
	{
		o_ptr = &inventory[item];
	}

	/* Get the item (on the floor) */
	else
	{
		o_ptr = &o_list[0 - item];
	}


	/* Take a turn */
	p_ptr->energy_use = 100;

	/* Not identified yet */
	ident = FALSE;

	/* Object level */
	lev = k_info[o_ptr->k_idx].level;

	/* Read the scroll */
	used_up = use_object(o_ptr, &ident);

	/* Combine / Reorder the pack (later) */
	p_ptr->notice |= (PN_COMBINE | PN_REORDER);

	/* The item was tried */
	object_tried(o_ptr);

	/* An identification was made */
	if (ident && !object_aware_p(o_ptr))
	{
		object_aware(o_ptr);
		gain_exp((lev + (p_ptr->lev / 2)) / p_ptr->lev);
	}

	/* Window stuff */
	p_ptr->window |= (PW_INVEN | PW_EQUIP);


	/* Hack -- allow certain scrolls to be "preserved" */
	if (!used_up) return;


	/* Destroy a scroll in the pack */
	if (item >= 0)
	{
		inven_item_increase(item, -1);
		inven_item_describe(item);
		inven_item_optimize(item);
	}

	/* Destroy a scroll on the floor */
	else
	{
		floor_item_increase(0 - item, -1);
		floor_item_describe(0 - item);
		floor_item_optimize(0 - item);
	}
}







/*
 * Use a staff
 *
 * One charge of one staff disappears.
 *
 * Hack -- staffs of identify can be "cancelled".
 */
void do_cmd_use_staff(int h_item)
{
	int item, chance, lev;

	bool ident;

	object_type *o_ptr;

	bool use_charge;

	cptr q, s;


	/* Are we being called from "do_cmd_handle()? */
	if (h_item != INVEN_TOTAL)
	{
		item = h_item;
	}
	else
	{
		/* Restrict choices to staves */
		item_tester_tval = TV_STAFF;

		/* Get an item */
		q = "Use which staff? ";
		s = "You have no staff to use.";
		if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
	}

	/* Get the item (in the pack) */
	if (item >= 0)
	{
		o_ptr = &inventory[item];
	}

	/* Get the item (on the floor) */
	else
	{
		o_ptr = &o_list[0 - item];
	}

	/* Take a turn */
	p_ptr->energy_use = 100;

	/* Not identified yet */
	ident = FALSE;

	/* Extract the item level */
	lev = k_info[o_ptr->k_idx].level;

	/* Base chance of success */
	chance = p_ptr->skill_dev;

	/* Confusion hurts skill */
	if (p_ptr->confused) chance = chance / 2;

	/* High level objects are harder */
	chance = chance - ((lev > 50) ? 50 : lev);

	/* Give everyone a (slight) chance */
	if ((chance < USE_DEVICE) && (rand_int(USE_DEVICE - chance + 1) == 0))
	{
		chance = USE_DEVICE;
	}

	/* Roll for usage */
	if ((chance < USE_DEVICE) || (randint(chance) < USE_DEVICE))
	{
		if (flush_failure) flush();
		msg_print("You failed to use the staff properly.");
		return;
	}

	/* Notice empty staffs */
	if (o_ptr->pval <= 0)
	{
		if (flush_failure) flush();
		msg_print("The staff has no charges left.");
		o_ptr->ident |= (IDENT_EMPTY);
		p_ptr->notice |= (PN_COMBINE | PN_REORDER);
		p_ptr->window |= (PW_INVEN);
		return;
	}


	/* Sound */
	sound(MSG_USE_STAFF);


	/* Use the staff */
	use_charge = use_object(o_ptr, &ident);


	/* Combine / Reorder the pack (later) */
	p_ptr->notice |= (PN_COMBINE | PN_REORDER);

	/* Tried the item */
	object_tried(o_ptr);

	/* An identification was made */
	if (ident && !object_aware_p(o_ptr))
	{
		object_aware(o_ptr);
		gain_exp((lev + (p_ptr->lev / 2)) / p_ptr->lev);
	}

	/* Window stuff */
	p_ptr->window |= (PW_INVEN | PW_EQUIP);


	/* Hack -- some uses are "free" */
	if (!use_charge) return;


	/* Use a single charge */
	o_ptr->pval--;

	/* Describe charges in the pack */
	if (item >= 0)
	{
		inven_item_charges(item);
	}

	/* Describe charges on the floor */
	else
	{
		floor_item_charges(0 - item);
	}
}


/*
 * Aim a wand (from the pack or floor).
 *
 * Use a single charge from a single item.
 * Handle "unstacking" in a logical manner.
 *
 * For simplicity, you cannot use a stack of items from the
 * ground.  This would require too much nasty code.
 *
 * There are no wands which can "destroy" themselves, in the inventory
 * or on the ground, so we can ignore this possibility.  Note that this
 * required giving "wand of wonder" the ability to ignore destruction
 * by electric balls.
 *
 * All wands can be "cancelled" at the "Direction?" prompt for free.
 *
 * Note that the basic "bolt" wands do slightly less damage than the
 * basic "bolt" rods, but the basic "ball" wands do the same damage
 * as the basic "ball" rods.
 */
void do_cmd_aim_wand(int h_item)
{
	int item, lev;

	bool ident;

	object_type *o_ptr;

	cptr q, s;


	/* Are we being called from "do_cmd_handle()? */
	if (h_item != INVEN_TOTAL)
	{
		item = h_item;
	}
	else
	{
		/* Restrict choices to wands */
		item_tester_tval = TV_WAND;

		/* Get an item */
		q = "Aim which wand? ";
		s = "You have no wand to aim.";
		if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
	}

	/* Get the item (in the pack) */
	if (item >= 0)
	{
		o_ptr = &inventory[item];
	}

	/* Get the item (on the floor) */
	else
	{
		o_ptr = &o_list[0 - item];
	}

	/* Aim the wand */
	if (!use_object(o_ptr, &ident)) return;


	/* Combine / Reorder the pack (later) */
	p_ptr->notice |= (PN_COMBINE | PN_REORDER);

	/* Mark it as tried */
	object_tried(o_ptr);

	/* Object level */
	lev = k_info[o_ptr->k_idx].level;

	/* Apply identification */
	if (ident && !object_aware_p(o_ptr))
	{
		object_aware(o_ptr);
		gain_exp((lev + (p_ptr->lev / 2)) / p_ptr->lev);
	}

	/* Window stuff */
	p_ptr->window |= (PW_INVEN | PW_EQUIP);


	/* Use a single charge */
	o_ptr->pval--;

	/* Describe the charges in the pack */
	if (item >= 0)
	{
		inven_item_charges(item);
	}

	/* Describe the charges on the floor */
	else
	{
		floor_item_charges(0 - item);
	}
}





/*
 * Activate (zap) a Rod
 *
 * Unstack fully charged rods as needed.
 *
 * Hack -- rods of perception can be "cancelled"
 * All rods can be cancelled at the "Direction?" prompt
 */
void do_cmd_zap_rod(int h_item)
{
	int item;
	bool ident;
	object_type *o_ptr;
	cptr q, s;


	/* Are we being called from "do_cmd_handle()? */
	if (h_item != INVEN_TOTAL)
	{
		item = h_item;
	}
	else
	{
		/* Restrict choices to rods */
		item_tester_tval = TV_ROD;

		/* Get an item */
		q = "Zap which rod? ";
		s = "You have no rod to zap.";
		if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
	}

	/* Get the item (in the pack) */
	if (item >= 0)
	{
		o_ptr = &inventory[item];
	}

	/* Get the item (on the floor) */
	else
	{
		o_ptr = &o_list[0 - item];
	}

	/* Zap the rod */
	if (!use_object(o_ptr, &ident)) return;

	/* Combine / Reorder the pack (later) */
	p_ptr->notice |= (PN_COMBINE | PN_REORDER);

	/* Tried the object */
	object_tried(o_ptr);

	/* Successfully determined the object function */
	if (ident && !object_aware_p(o_ptr))
	{
		/* Object level */
		int lev = k_info[o_ptr->k_idx].level;

		object_aware(o_ptr);
		gain_exp((lev + (p_ptr->lev / 2)) / p_ptr->lev);
	}

	/* Window stuff */
	p_ptr->window |= (PW_INVEN | PW_EQUIP);
}




/*
 * Hook to determine if an object is activatable
 */
static bool item_tester_hook_activate(const object_type *o_ptr)
{
	u32b f1, f2, f3;

	/* Not known */
	if (!object_known_p(o_ptr)) return (FALSE);

	/* Extract the flags */
	object_flags(o_ptr, &f1, &f2, &f3);

	/* Check activation flag */
	if (f3 & (TR3_ACTIVATE)) return (TRUE);

	/* Assume not */
	return (FALSE);
}


/*
 * Activate a wielded object.  Wielded objects never stack.
 * And even if they did, activatable objects never stack.
 *
 * Note that it always takes a turn to activate an artifact, even if
 * the user hits "escape" at the "direction" prompt.
 */
void do_cmd_activate(int h_item)
{
	int item, lev, chance;
	bool ident;
	object_type *o_ptr;

	cptr q, s;


	/* Are we being called from "do_cmd_handle()? */
	if (h_item != INVEN_TOTAL)
	{
		item = h_item;
	}
	else
	{
		/* Prepare the hook */
		item_tester_hook = item_tester_hook_activate;

		/* Get an item */
		q = "Activate which item? ";
		s = "You have nothing to activate.";
		if (!get_item(&item, q, s, (USE_EQUIP))) return;
	}

	/* Get the item (in the pack) */
	if (item >= 0)
	{
		o_ptr = &inventory[item];
	}

	/* Get the item (on the floor) */
	else
	{
		o_ptr = &o_list[0 - item];
	}


	/* Take a turn */
	p_ptr->energy_use = 100;

	/* Extract the item level */
	lev = k_info[o_ptr->k_idx].level;

	/* Hack -- use artifact level instead */
	if (artifact_p(o_ptr)) lev = a_info[o_ptr->name1].level;

	/* Base chance of success */
	chance = p_ptr->skill_dev;

	/* Confusion hurts skill */
	if (p_ptr->confused) chance = chance / 2;

	/* High level objects are harder */
	chance = chance - ((lev > 50) ? 50 : lev);

	/* Give everyone a (slight) chance */
	if ((chance < USE_DEVICE) && (rand_int(USE_DEVICE - chance + 1) == 0))
	{
		chance = USE_DEVICE;
	}

	/* Roll for usage */
	if ((chance < USE_DEVICE) || (randint(chance) < USE_DEVICE))
	{
		if (flush_failure) flush();
		msg_print("You failed to activate it properly.");
		return;
	}

	/* Activate the object */
	(void)use_object(o_ptr, &ident);
}

/*
 * Allow user to "prevent" certain choices.
 *
 * This is a slightly modified copy of get_item_allow in object1.c
 * This is used when 'h'andling items. The use has already confirmed
 * for any !* or !h inscription; now we check for the inscription
 * that fits the object type (e.g. when handling a scroll, check !r)
 */
bool handle_item_allow(int item, s16b command)
{
	cptr s;

	object_type *o_ptr;

	/* Inventory */
	if (item >= 0)
	{
		o_ptr = &inventory[item];
	}

	/* Floor */
	else
	{
		o_ptr = &o_list[0 - item];
	}

	/* No inscription */
	if (!o_ptr->note) return (TRUE);

	/* Find a '!' */
	s = strchr(quark_str(o_ptr->note), '!');

	/* Process preventions */
	while (s)
	{
		/* Check the "restriction" */
		if (s[1] == command)
		{
			/* Verify the choice */
			if (!verify_item("Really try", item)) return (FALSE);
		}

		/* Find another '!' */
		s = strchr(s + 1, '!');
	}

	/* Allow it */
	return (TRUE);
}


/*
 * Hook to determine if an object can be handled
 *
 * Accept all items on floor and in inventory, but only activateable items in equipment
 */
static bool item_tester_hook_handle(const object_type *o_ptr)
{
	/* The procedure to determining if an item is equipped is a big hack */
	/* Unfortunately, we have no choice since we can't tell from the object info */

	/* Determine where the object kind is normally equipped */
	s16b slot = wield_slot(o_ptr);

	/* If the object kind is equippable at all, then compare item in equipment slot */
	/* with item to test - if the pointers are equal, it's the same item */
	if ((slot >= INVEN_WIELD) && (&inventory[slot] == o_ptr))
	{
		/* The item is equipped. Test if players knows that it can be activated. */
		u32b f1, f2, f3;

		/* Not known */
		if (!object_known_p(o_ptr)) return (FALSE);

		/* Extract the flags */
		object_flags(o_ptr, &f1, &f2, &f3);

		/* Check activation flag */
		if (f3 & (TR3_ACTIVATE)) return (TRUE);

		/* Assume not */
		return (FALSE);
	}
	/* All unequipped items are all okay */
	else return (TRUE);
}


/*
 * Handle an item (generic use command)
 *
 * Accepts all items in inventory, equipment or floor;
 * calls command handler to execute most appropriate command
 * for the object kind.
 */
void do_cmd_handle_item(void)
{
	int item, lev, chance;
	bool ident;
	object_type *o_ptr;

	cptr q, s;


	/* Prepare the hook */
	item_tester_hook = item_tester_hook_handle;

	/* Get an item */
	q = "Handle which item? ";
	s = "You have nothing to handle.";
	if (!get_item(&item, q, s, (USE_EQUIP | USE_INVEN | USE_FLOOR))) return;

	/* Get the item (in the pack) */
	if (item >= 0)
	{
		o_ptr = &inventory[item];
	}

	/* Get the item (on the floor) */
	else
	{
		o_ptr = &o_list[0 - item];
	}


	/* Analyze the object and determine appropriate action */
	switch (o_ptr->tval)
	{
		/* Junk items will be destroyed */
		case TV_SKELETON:
		case TV_BOTTLE:
		case TV_JUNK:
		{
			/* Verify use if inscribed with !k */
			if (!handle_item_allow(item, 'k')) return;

			/* Destroy the junk */
			do_cmd_destroy(item);

			return;
		}

		/* Spikes will be used to jam a door */
		case TV_SPIKE:
		{
			/* Verify use if inscribed with !j */
			if (!handle_item_allow(item, 'j')) return;

			/* Spike the door */
			do_cmd_spike(item);

			return;
		}

		/* Chests will be destroyed if empty, dropped if held, disarmed or opened if on floor */
		case TV_CHEST:
		{
			/* Chest already empty? */
			if (!o_ptr->pval)					/* no items in chest */
			{
				/* Verify use if inscribed with !k */
				if (!handle_item_allow(item, 'k')) return;

				/* Destroy the empty chest */
				do_cmd_destroy(item);

				return;
			}

			/* Chest currently in the pack? */
			if (item >= 0)
			{
				/* Verify use if inscribed with !d */
				if (!handle_item_allow(item, 'd')) return;

				/* Drop the chest */
				do_cmd_drop(item);

				return;
			}

			/* Any known traps on this chest? */
			if ((o_ptr->pval > 0) &&			/* chest must not yet be disarmed */
			    (chest_traps[o_ptr->pval]) &&	/* chest must actually be trapped */
			    (object_known_p(o_ptr)))		/* trap must already be found */
			{
				/* Verify use if inscribed with !D */
				if (!handle_item_allow(item, 'D')) return;

				/* Disarm whatever is on the player location */
				do_cmd_disarm(5);

				return;
			}

			/* Not empty, not in pack, no (known) traps - try to open */
			/* Verify use if inscribed with !o */
			if (!handle_item_allow(item, 'o')) return;

			/* Open whatever is on the player location */
			do_cmd_open(5);

			return;
		}

		/* Ammunition will be fired if correct launcher is equipped, thrown otherwise */
		case TV_SHOT:
		case TV_ARROW:
		case TV_BOLT:
		{
			/* Proper type of launcher equipped? */
			if (o_ptr->tval == p_ptr->ammo_tval)
			{
				/* Verify use if inscribed with !f */
				if (!handle_item_allow(item, 'f')) return;

				/* Fire the ammunition */
				do_cmd_fire(item);

				return;
			}

			/* Wielded launcher unusable for this ammo - throw it instead */
			/* Verify use if inscribed with !v */
			if (!handle_item_allow(item, 'v')) return;

			/* Throw the ammunition */
			do_cmd_throw(item);

			return;
		}

		/* Equippable items: mostly activate if currently equipped, wear/wield otherwise */
		/* Exception for torches and lanterns: refill if same equipped, wear/wield otherwise */
		case TV_BOW:
		case TV_DIGGING:
		case TV_HAFTED:
		case TV_POLEARM:
		case TV_SWORD:
		case TV_BOOTS:
		case TV_GLOVES:
		case TV_HELM:
		case TV_CROWN:
		case TV_SHIELD:
		case TV_CLOAK:
		case TV_SOFT_ARMOR:
		case TV_HARD_ARMOR:
		case TV_DRAG_ARMOR:
		case TV_LITE:
		case TV_AMULET:
		case TV_RING:
		{
			/* If item is equipped, try to activate it */
			/* (Note - equipped items without activation can't be handled, see item_tester_hook_handle */
			/*  This also means that we don't have to test for torches or lanterns here) */
			if (item >= INVEN_WIELD)
			{
				/* Verify use if inscribed with !A */
				if (!handle_item_allow(item, 'A')) return;

				/* Activate the item */
				do_cmd_activate(item);

				return;
			}

			/* If item is torch or lantern and same object is equipped, use it to refill */
			if ((o_ptr->tval == TV_LITE) &&
			    ((o_ptr->sval == SV_LITE_LANTERN) || (o_ptr->sval == SV_LITE_TORCH)))
			{
				object_type *lite_ptr;

				/* Get current light */
				lite_ptr = &inventory[INVEN_LITE];

				/* Check if light is used, and if type is the same */
				if ((lite_ptr->tval == TV_LITE) && (lite_ptr->sval == o_ptr->sval))
				{
					/* Verify use if inscribed with !F */
					if (!handle_item_allow(item, 'F')) return;

					/* Refill the light source */
					do_cmd_refill(item);

					return;
				}
			}

			/* Not equipped, not a torch or lantern that can be used to refill - then wear/wield it */
			/* Verify use if inscribed with !w */
			if (!handle_item_allow(item, 'w')) return;

			/* Wear/wield the equipment */
			do_cmd_wield(item);

			return;
		}

		/* Staves will be used */
		case TV_STAFF:
		{
			/* Verify use if inscribed with !u */
			if (!handle_item_allow(item, 'u')) return;

			/* Use the staff */
			do_cmd_use_staff(item);

			return;
		}

		/* Wands will be aimed */
		case TV_WAND:
		{
			/* Verify use if inscribed with !a */
			if (!handle_item_allow(item, 'a')) return;

			/* Aim the wand */
			do_cmd_aim_wand(item);

			return;
		}

		/* Rods will be zapped */
		case TV_ROD:
		{
			/* Verify use if inscribed with !z */
			if (!handle_item_allow(item, 'z')) return;

			/* Zap the rod */
			do_cmd_zap_rod(item);

			return;
		}

		/* Scrolls will be read */
		case TV_SCROLL:
		{
			/* Verify use if inscribed with !r */
			if (!handle_item_allow(item, 'r')) return;

			/* Read the scroll */
			do_cmd_read_scroll(item);

			return;
		}

		/* Potions will be quaffed, except Potion of Detonations, which is thrown */
		case TV_POTION:
		{
			/* Check for (known) Potion of Detonations */
			if ((o_ptr->sval == SV_POTION_DETONATIONS) &&
			    (object_aware_p(o_ptr)))
			{
				/* Verify use if inscribed with !v */
				if (!handle_item_allow(item, 'v')) return;

				/* Throw the exploding potion */
				do_cmd_throw(item);

				return;
			}

			/* Not Potion of Detonations or not yet aware - quaff it */
			/* Verify use if inscribed with !q */
			if (!handle_item_allow(item, 'q')) return;

			/* Quaff the potion */
			do_cmd_quaff_potion(item);

			return;
		}

		/* Flask of Oil: Use to refill lantern if one is equipped, throw otherwise */
		case TV_FLASK:
		{
			object_type *lite_ptr;

			/* Get current light */
			lite_ptr = &inventory[INVEN_LITE];

			/* Check if a lantern is used */
			if ((lite_ptr->tval == TV_LITE) && (lite_ptr->sval == SV_LITE_LANTERN))
			{
				/* Verify use if inscribed with !F */
				if (!handle_item_allow(item, 'F')) return;

				/* Refill the light source */
				do_cmd_refill(item);

				return;
			}

			/* No lantern equipped - throw the oil */
			/* Verify use if inscribed with !v */
			if (!handle_item_allow(item, 'v')) return;

			/* Throw the oil */
			do_cmd_throw(item);

			return;
		}

		/* Food will be eaten, except Mushrooms of Unhealth / of Disease, which are thrown */
		case TV_FOOD:
		{
			/* Check for (known) Mushroom of Unhealth or Mushroom of Disease */
			if (((o_ptr->sval == SV_FOOD_UNHEALTH) || (o_ptr->sval == SV_FOOD_DISEASE)) &&
			    (object_aware_p(o_ptr)))
			{
				/* Verify use if inscribed with !v */
				if (!handle_item_allow(item, 'v')) return;

				/* Throw the exploding potion */
				do_cmd_throw(item);

				return;
			}

			/* Not Mushroom of Unhealth/Disease or not yet aware - eat it */
			/* Verify use if inscribed with !E */
			if (!handle_item_allow(item, 'E')) return;

			/* Eat the food */
			do_cmd_eat_food(item);

			return;
		}

		/* Spell books: cast a spell if you're a spell caster, destroy otherwise */
		case TV_MAGIC_BOOK:
		{
			/* Is the player able to cast spells? */
			if (cp_ptr->spell_book == TV_MAGIC_BOOK)
			{
				/* Verify use if inscribed with !m */
				if (!handle_item_allow(item, 'm')) return;

				/* Cast a spell from this book */
				do_cmd_cast(item);

				return;
			}

			/* Player can't cast spells - destroy the book */
			/* Verify use if inscribed with !k */
			if (!handle_item_allow(item, 'k')) return;

			/* Destroy the book */
			do_cmd_destroy(item);

			return;
		}

		/* Prayer books: recite a prayer if you're a priest/paladin, destroy otherwise */
		case TV_PRAYER_BOOK:
		{
			/* Is the player able to recite prayers? */
			if (cp_ptr->spell_book == TV_PRAYER_BOOK)
			{
				/* Verify use if inscribed with !p */
				if (!handle_item_allow(item, 'p')) return;

				/* Recite a prayer from this book */
				do_cmd_pray(item);

				return;
			}

			/* Player can't recite prayers - destroy the book */
			/* Verify use if inscribed with !k */
			if (!handle_item_allow(item, 'k')) return;

			/* Destroy the book */
			do_cmd_destroy(item);

			return;
		}

		/* Paranoia */
		default:
		{
			if (flush_failure) flush();
			msg_print("You can't handle it.");
			return;
		}
	}
}
